% User Interface for Network Component Mapping Algorithm - NCM sampling algorithm described in 
%   Brynildsen MP, Wu TY, Jang SS, JC Liao "Biological Network Mapping and Source Signal Deduction"
%   
% 
% function [network] = NetworkComponentMapping
%    
%
%     Output
%     ----------
%       network: NCM deduced network from data
%       
%
% NOTE: The expression data file must be a single matrix is ASCII format
%    
% ------------------------------------------------------------------------------------------
% Last revision March 6th, 2007 via MP Brynildsen, UCLA
% -------------------------------------------------------------------------
% -----------------

function [network] = NetworkComponentMapping
fprintf('Please make sure all data files are within your\ncurrent directory');

%Indicate Approximate Noise Level of Data
Choice1 = 'High';
Choice2 = 'Low';
either(1) = cellstr(Choice1);
either(2) = cellstr(Choice2);
[Selection,ok] = listdlg('PromptString','Level of noise in your data?',...
                                'SelectionMode','single',...
                                'ListString',either);

if (size(Selection,1) == 0), %Cancel button
    fprintf('\n\nProgram CANCELLED by user\n\n');
    stop
end
if (Selection == 1), %high level
    def = {'500','100','0.02','0.25'};
    noise_level = 1;
end
if (Selection == 2), %low level
    def = {'100','50','0.01','0.05'};
    noise_level = 2;
end

                            
%Specify Thresholds
prompt = {'Enter number of NCM iterations:','Enter number of paths','Enter NullMap threshold:','Enter Recursive Algorithm threshold:'};
dlg_title = 'NCM characteristics';
num_lines= 1;
answer  = inputdlg(prompt,dlg_title,num_lines,def);
characteristics = str2num(char(answer));
if (min(size(characteristics)) == 0),
    fprintf('\n\nThe number of regulators has to be numeric\n\n');
    stop
end

%Specify NullMap criterion
Choice1 = 'Strict';
Choice2 = 'Relaxed';
Choice3 = 'Released';
either(1) = cellstr(Choice1);
either(2) = cellstr(Choice2);
either(3) = cellstr(Choice3);
[Selection_1,ok] = listdlg('PromptString','Choose initial mapping criterion',...
                                'SelectionMode','single',...
                                'ListString',either);                

if (size(Selection_1,1) == 0), %Cancel button
    fprintf('\n\nProgram CANCELLED by user\n\n');
    stop
end
if (Selection_1 == 1),%Strict Iterations
    type = 0;
end  
if (Selection_1 == 2),%Relaxed Iterations
    type = 1;
end
if (Selection_1 == 3),%Released Iterations
    type = 2;
end

%Load the data
fprintf('\n\n NOTE: Expression data must be a single matrix in ASCII format!!!\n\n');
def = {''};
prompt = {'Enter name of data file:'};
dlg_title = 'Load Data';
num_lines= 1;
answer  = inputdlg(prompt,dlg_title,num_lines,def);
data_file = answer{1};
DATA = load(data_file);
[N,M] = size(DATA);


%Specify number of regulators                    
prompt = {'Specify the number of regulators: (if unknown use 0)'};
dlg_title = 'Regulators';
num_lines= 1;
def = {'0'};
answer  = inputdlg(prompt,dlg_title,num_lines,def);
reg_value = str2num(char(answer));
if (min(size(reg_value)) == 0),
    fprintf('\n\nThe number of regulators has to be numeric\n\n');
    stop
end
if (reg_value == 0),%Unknown number of regulators
    singularvalues = svd(DATA);
    singularvalues = round(1000*singularvalues/sum(singularvalues));
    if (noise_level == 1),
        for jj = 1:M,
            if (singularvalues(jj) < 50) %use a minimum of 5%
                reg_value = jj - 1;
                fprintf('\n\nNumber of regulators determined: %.0f\n\n',reg_value);
                break
            end
        end
    end
    if (noise_level == 2),
        for jj = 1:M,
            if (singularvalues(jj) < 10) %use a minimum of 1%
                reg_value = jj - 1;
                fprintf('Number of regulators determined: %.0f\n\n',reg_value);
                break
            end
        end
    end
end  

%Perform NCM
[SaveIt] = multipleRuns_NCM(DATA,reg_value,characteristics(3),characteristics(4),type,characteristics(2),characteristics(1));

%Process NCM output
[k,t] = size(SaveIt);
characteristics(1)
for jj = 1:characteristics(1),
    x(jj,1) = max(size(find(SaveIt(:,jj) == 0)));
end

network_index = find(x == max(x))
network_norm = SaveIt(k,network_index)
min_norm = find(network_norm == min(network_norm))
ncm_network = network_index(min_norm(1));
network = reshape(SaveIt(1:k-1,ncm_network),N,reg_value);

